/*
   This program parses and then evaluates strings from Language_2.
*/

public class Language_2_Examples
{
   public static void main(String[] args)
   {
      String[] programs =
      {
         "( prog " +
           "( var y 6 ) " +
           "( fun ( f x ) ( * x y ) ) " +  // y is a non-local reference
           "( f 5 ) )",

         "( prog " +
           "( fun ( f x ) ( * x x ) ) " +
           "( fun ( g y ) ( + y ( f y ) ) ) " +
           "( g 5 ) )",

         "( prog " +
           "( var w 10 ) " +
           "( fun ( f x y z ) ( + x y z w ) ) " + // w is a non-local reference
           "( var x ( f 1 2 3 ) ) " +
           "( var y ( f 4 5 x ) ) " +
           "( var z ( f ( + w x y ) 10 20 ) ) )",

         "( prog " +
           "( var n 1 ) " +
           "( fun ( f x ) ( * n x ) ) " +
           "( fun ( g y ) ( var n 5 ) " +
                     "( f y ) ) " +
           "( g 2 ) )",  // result =  2 with static scope
                         // result = 10 with dynamic scope

         "( prog " +
           "( var n 1 ) " +
           "( fun ( f x ) ( * n x ) ) " +
           "( fun ( g x ) ( var n 5 ) " +
                     "( var r ( f x ) ) " +
                     "r ) " +
           "( g 2 ) )",  // result =  2 with static scope
                        // result = 10 with dynamic scope

         // recursive function
         "( prog " +
           "( fun ( fac n ) " +
              "( if ( <= n 0 ) " +
                   "1 " +
                   "( * n ( fac ( - n 1 ) ) ) ) ) " +
           "( fac 6 ) )",

         // recursive function
         "( prog " +
           "( fun ( even n ) " +
              "( && ( != n 1 ) " +       // use short-circuiting
                   "( || ( <= n 0 ) " +  // use short-circuiting
                        "( even ( - n 2 ) ) ) ) ) " +
           "( print ( even 7 ) ) " +
           "( print ( even 6 ) ) " +
           "( print ( even 0 ) ) " +
           "( even -1 ) )",

         // mutually recursive functions
         "( prog " +
           "( fun ( even n ) " +
              "( || ( <= n 0 ) " +       // use short-circuiting
                   "( odd ( - n 1 ) ) ) ) " +
           "( fun ( odd n ) " +
              "( && ( >  n 0 ) " +        // use short-circuiting
                   "( || ( == n 1 ) " +   // use short-circuiting
                        "( even ( - n 1 ) ) ) ) ) " +
           "( print ( even 8 ) ) " +
           "( print ( odd 7 ) ) " +
           "( print ( odd 6 ) ) " +
           "( print ( even 3 ) ) " +
           "( even -1 ) )",
      };


      int i = 0;
      for (i = 0; i < programs.length; i++)
      {
         System.out.println(i + " =============================================");

         try // to parse the program.
         {
            // Build the abstract syntax tree that represents the program.
            Tree ast = Parse.parse( programs[i] );
          //Tree ast = BuildTree.buildTree( programs[i] );

            // Pretty-print the abstract syntax tree.
            System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );

            // Print the infix version of the program.
            System.out.println( AST2infix.ast2infix( ast ) + "\n" );

            try // to evaluate the program (interpret the AST).
            {
               Value value = Evaluate.eval( ast );
               System.out.println( "result = " + value + "\n" );
            }
            catch (EvalException e)
            {
             //System.out.println( e );
               e.printStackTrace();
            }

            if(true)
            try // to create dot and png files from the AST.
            {
               // Create an empty dot file.
               String baseName = String.format("Language_2.%02d", i);
               java.io.PrintWriter out = new java.io.PrintWriter(
                                            new java.io.File(baseName + ".dot") );
               // Put dot commands into the dot file
               out.println( Tree2dot.tree2dot(ast) + "\n" );
               out.close();
               // Create a command line for running dot.exe.
               String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                               "-Tpng",
                               baseName + ".dot",
                               "-o",
                               baseName + ".png"};
               // Execute the command line.
               java.lang.Runtime.getRuntime().exec(cmd);
            }
            catch (Exception e)
            {
               System.out.println( e );
             //e.printStackTrace();
            }
         }
         catch (ParseException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
      }
   }//main()

}//Language_2_Examples
