/*
   This program parses and then evaluates strings from Language_2.

   These examples demonstrate two ways that a function can be
   used as a first class value.
   (1) A function can be passed as a parameter to another function.
   (2) A function can be returned by another function
*/

public class Language_2_Examples_2
{
   public static void main(String[] args)
   {
      String[] programs =
      {
         "( prog " +
           "( var y 5 ) " +
           "( fun ( f x ) ( * x x ) ) " +
           "( fun ( g h ) ( h y ) ) " + // y is a non-local reference
           "( set y 7 ) " +
           "( var r ( g f ) ) )", // returns 25 using static scope
                                  // (returns 49 using dynamic scope)

         "( prog " +
           "( fun ( f x ) ( * x x ) ) " +
           "( fun ( g x ) ( * x x x ) ) " +
           "( fun ( h x ) ( if x f g ) ) " + // returns either f or g
           "( var r1 ( h  true ) ) " +   // returns "f"
           "( var r2 ( h false ) ) " +   // returns "g"
           "( var r3 ( r1 5 ) ) " +      // returns 25
           "( var r4 ( r2 5 ) ) )",      // returns 125

         "( prog " +
           "( var y 2 ) " +
           "( fun ( f x ) ( * x y ) ) " +    // y is a non-local reference
           "( fun ( g x ) ( * x x y ) ) " +
           "( fun ( h x ) ( if x f g ) ) " + // returns either f or g
           "( set y 3 ) " +
           "( var k ( h true ) ) " +     // NOTICE HOW var AND fun ARE RELATED!
           "( var r1 ( k 5 ) ) " +       // returns 10 (or 15 using dyn scope)
           "( set k ( h false ) ) " +
           "( var r2 ( k 5 ) ) )"        // returns 50 (or 75 using dyn scope)
      };

      int i = 0;
      for (i = 0; i < programs.length; i++)
      {
         System.out.println(i + " =============================================");

         try // to parse the program.
         {
            // Build the abstract syntax tree that represents the program.
            Tree ast = Parse.parse( programs[i] );
          //Tree ast = BuildTree.buildTree( programs[i] );

            // Pretty-print the abstract syntax tree.
            System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );

            // Print the infix version of the program.
            System.out.println( AST2infix.ast2infix( ast ) + "\n" );

            try // to evaluate the program (interpret the AST).
            {
               Value value = Evaluate.eval( ast );
               System.out.println( "result = " + value + "\n" );
            }
            catch (EvalException e)
            {
             //System.out.println( e );
               e.printStackTrace();
            }

            if(true)
            try // to create dot and png files from the AST.
            {
               // Create an empty dot file.
               String baseName = String.format("Language_2_2.%02d", i);
               java.io.PrintWriter out = new java.io.PrintWriter(
                                            new java.io.File(baseName + ".dot") );
               // Put dot commands into the dot file
               out.println( Tree2dot.tree2dot(ast) + "\n" );
               out.close();
               // Create a command line for running dot.exe.
               String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                               "-Tpng",
                               baseName + ".dot",
                               "-o",
                               baseName + ".png"};
               // Execute the command line.
               java.lang.Runtime.getRuntime().exec(cmd);
            }
            catch (Exception e)
            {
               System.out.println( e );
             //e.printStackTrace();
            }
         }
         catch (ParseException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
      }
   }//main()

}//Language_2_Examples_2
