/*
   This file evaluates the following Language_2 program.
      ( prog
        ( var w 5 )
        ( fun ( f x y z ) ( * x y z w ) )
        ( var x 4 )
        ( var y 3 )
        ( f y x ( + x y ) ) )
   Notice that the function f has a non-local reference to
   the global variable w.
*/

public class Language_2_First_Example
{
   public static void main(String[] args)
   {
      String program =
         "( prog " +
           "( var w 5 ) " +
           "( fun ( f x y z ) ( * x y z w ) ) " +  // w is a non-local reference
           "( var x 4 ) " +
           "( var y 3 ) " +
           "( f y x ( + x y ) ) )";

      try // to parse the program.
      {
         // Build the abstract syntax tree that represents the program.
         Tree ast = Parse.parse( program );
       //Tree ast = BuildTree.buildTree( program );

         // Pretty-print the abstract syntax tree.
         System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );

         // Print the infix version of the program.
         System.out.println( AST2infix.ast2infix( ast ) + "\n" );

         try // to evaluate the program (interpret the AST).
         {
            Value value = Evaluate.eval( ast );
            System.out.println( "result = " + value + "\n" );
         }
         catch (EvalException e)
         {
          //System.out.println( e );
            e.printStackTrace();
         }

         try // to create dot and png files from the AST.
         {
            // Create an empty dot file.
            String baseName = "Language_2_First_Example";
            java.io.PrintWriter out = new java.io.PrintWriter(
                                         new java.io.File(baseName + ".dot") );
            // Put dot commands into the dot file
            out.println( Tree2dot.tree2dot(ast) + "\n" );
            out.close();
            // Create a command line for running dot.exe.
            String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
          //e.printStackTrace();
         }
      }
      catch (ParseException e)
      {
         System.out.println(e);
       //e.printStackTrace();
      }
   }//main()

}//Language_2_First_Example
